@@ -11,8 +11,7 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
         for (i in 1:length(stage)) if (!(stage[i] %in% c("stratified", 
             "cluster", ""))) 
             stop("the stage argument is wrong")
-    }
-    else number = length(size)
+    }     else number = length(size)
     if (number > 1) {
         if (!missing(varnames)) {
             if (!is.list(size)) 
@@ -21,18 +20,20 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
             varnames = as.list(varnames)
             size1 = size[[1]]
             varnames1 = varnames[[1]]
-            if (method %in% c("systematic", "poisson")) 
+            if (method[[1]] %in% c("systematic", "poisson")) 
                 pik1 = pik[[1]]
         }
         else {
             size1 = size[[1]]
             varnames1 = NULL
-            if (method %in% c("systematic", "poisson")) 
+            if (method[[1]] %in% c("systematic", "poisson")) 
                 pik1 = pik[[1]]
         }
     }
     else {
         size1 = size
+        if(missing(method)) method="srswor"
+        else
         if (method %in% c("systematic", "poisson")) 
             pik1 = pik
     }
@@ -41,35 +42,35 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
     if (missing(stage)) {
         if (missing(varnames)) 
             if (missing(method)) 
+                s = strata(data, stratanames = NULL, size = size1, description)
+             else if (method[[1]] %in% c("systematic", "poisson")) 
                 s = strata(data, stratanames = NULL, size = size1, 
-                  description)
-            else if (method %in% c("systematic", "poisson")) 
-                s = strata(data, stratanames = NULL, size = size1, 
-                  method, pik = pik1, description)
+                  method[[1]], pik = pik1, description)
             else s = strata(data, stratanames = NULL, size = size1, 
-                method, description)
-        else s = strata(data, stratanames = NULL, size1, method, 
+                method[[1]], description)
+        else s = strata(data, stratanames = NULL, size1, method[[1]], 
             pik = pik1, description)
     }
     else if (stage[1] == "stratified") {
-        s = strata(data, varnames1, size1, method, pik = pik1, 
-            description)
+        s = strata(data, varnames1, size1, method="srswor",description)
         dimension_st = table(s$Stratum)
-    }
+        if(description) cat("Number of strata:",length(dimension_st),"\n")  
+                                       }
     else {
-        s = cluster(data, varnames1, size1, method, pik1, description)
+        s = cluster(data, varnames1, size1, method[[1]], pik1, description)
         if (is.null(s)) 
             stop("0 selected units in the first stage")
         m = match(varnames1, names(s))
         nl = nlevels(as.factor(s[, m]))
         lev = levels(as.factor(s[, m]))
         if (nl >= 1) {
-            dimension_cl = numeric(nl)
-            for (i in 1:nl) dimension_cl[i] = nrow(subset(s, 
-                as.factor(s[, m]) == lev[i]))
-        }
+            dimension_cl = NULL
+            for (i in 1:nl) 
+                   if(nrow(subset(s,s[, m] == unique(s[,m])[i]))>0)
+                                dimension_cl = c(dimension_cl,nrow(subset(s,s[, m] == unique(s[,m])[i])))
+                     }
         dimension = dimension_cl
-    }
+          }
     if (is.null(s)) 
         stop("0 selected units in the first stage")
     if (number > 1) 
@@ -96,23 +97,24 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                           dimension_st[ii]), ]
                         r = getdata(data, r)
                         m = match(varnames[[j]], names(r))
-                        if (method %in% c("systematic", "poisson")) {
+                        if (method[[j]] %in% c("systematic", "poisson")) {
                           index = res[[j - 1]][(limit + 1):(limit + 
                             dimension_st[ii]), ]$ID_unit
                           pikk = pik[[j]][index]
                           if (!is.null(r)) 
                             s3 = cluster(r, clustername = varnames[[j]], 
-                              size = size[[j]][ii], method = method, 
+                              size = size[[j]][ii], method = method[[j]], 
                               pik = pikk, description)
                           else s3 = NULL
                         }
                         else {
+
                           s3 = cluster(r, clustername = varnames[[j]], 
-                          size = size[[j]][ii], method = method, 
-                          description = description)
-                              }
+                            size = size[[j]][ii], method = method[[j]], 
+                            description = description)
+                        }
                         limit = limit + dimension_st[ii]
-                        if (method == "srswr") {
+                        if (method[[j]] == "srswr") {
                           s3 = cbind.data.frame(r[s3$ID_unit, 
                             m], r[s3$ID_unit, ]$ID_unit, s3$Replicates, 
                             s3$Prob, r[s3$ID_unit, ]$Prob * s3$Prob)
@@ -120,7 +122,7 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                             "Replicates", paste("Prob_", j, "_stage"), 
                             "Prob")
                         }
-                        else if (!is.null(s3)) {
+                        else if(!is.null(s3)) {
                           s3 = cbind.data.frame(r[s3$ID_unit, 
                             m], r[s3$ID_unit, ]$ID_unit, s3$Prob, 
                             result[s3$ID_unit, ]$Prob * s3$Prob)
@@ -137,8 +139,17 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                       }
                   }
                   else if (stage[[j - 1]] == "cluster") {
-                    dimension_cl=size[[j-1]]
-                    k = length(dimension_cl)
+                 m_cl = match(varnames, names(res[[j-1]]),0)
+                 mat=res[[j-1]][, m_cl]
+                 nl = nlevels(as.factor(mat))
+                 if (nl >= 1)     {
+            dimension_cl =NULL
+            for (i in 1:nl) 
+               if(length(subset(mat, mat==unique(mat)[i]))>0)
+                  dimension_cl = c(dimension_cl,length(subset(mat, mat==unique(mat)[i])))
+            k = length(dimension_cl)                   
+                  }
+            else stop("error in the previous stage")
                     s1 = NULL
                     limit = 0
                     dimension = list()
@@ -146,8 +157,7 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                       warning("the number of selected clusters in the previous stage is larger than the size argument")
                       warning("the size 1 is added")
                       size1 = size[[j]]
-                      for (i in 1:(k - length(size[[j]]))) size1 = c(size1, 
-                        1)
+                      for (i in 1:(k - length(size[[j]]))) size1 = c(size1, 1)
                     }
                     else size1 = size[[j]]
                     if (k >= 1) 
@@ -156,7 +166,7 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                           dimension_cl[ii]), ]
                         r = getdata(data, r)
                         m = match(varnames[[j]], names(r))
-                        if (method %in% c("systematic", "poisson")) {
+                        if (method[[j]] %in% c("systematic", "poisson")) {
                           m1 = match(varnames[[j - 1]], names(r))
                           m2 = match(varnames[[j - 1]], names(data))
                           mm = match(r[1, m1], levels(factor(data[, 
@@ -164,15 +174,15 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                           pikk = as.numeric(pik[[j]][[mm]])
                           if (!is.null(r)) 
                             s3 = cluster(r, clustername = varnames[[j]], 
-                              size = size1[[ii]], method = method, 
+                              size = size1[[ii]], method = method[[j]], 
                               pik = pikk, description)
                           else s3 = NULL
                         }
                         else s3 = cluster(r, clustername = varnames[[j]], 
-                          size = size1[ii], method = method, 
+                          size = size1[ii], method = method[[j]], 
                           pik, description)
                         limit = limit + dimension_cl[ii]
-                        if (method == "srswr") {
+                        if (method[[j]] == "srswr") {
                           s3 = cbind.data.frame(r[s3$ID_unit, 
                             m], r[s3$ID_unit, ]$ID_unit, s3$Replicates, 
                             s3$Prob, r[s3$ID_unit, ]$Prob * s3$Prob)
@@ -206,8 +216,7 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                     warning("the number of selected clusters at the previous stage is larger than the size argument")
                     warning("the size 1 is added")
                     size1 = size[[j]]
-                    for (i in 1:(k - length(size[[j]]))) size1 = c(size1, 
-                      1)
+                    for (i in 1:(k - length(size[[j]]))) size1 = c(size1,1)
                   }
                   else size1 = size[[j]]
                   if (k >= 1) 
@@ -215,19 +224,19 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                       r = res[[j - 1]][(limit + 1):(limit + dimension[[i]][ii]), 
                         ]
                       count = count + 1
-                      if (method %in% c("systematic", "poisson")) {
+                      if (method[[j]] %in% c("systematic", "poisson")) {
                         index = res[[j - 1]][(limit + 1):(limit + 
                           dimension[[i]][ii]), ]$ID_unit
                         pikk = pik[[j]][index]
                         if (!is.null(r)) 
                           s2 = strata(r, NULL, size = size1[count], 
-                            method = method, pik = pikk, description)
+                            method = method[[j]], pik = pikk, description)
                         else s2 = NULL
                       }
                       else s2 = strata(r, NULL, size = size1[count], 
-                        method = method, pik, description)
+                        method = method[[j]], pik, description)
                       limit = limit + dimension[[i]][ii]
-                      if (method == "srswr") {
+                      if (method[[j]] == "srswr") {
                         s2 = cbind.data.frame(r[s2$ID_unit, ]$ID_unit, 
                           s2$Replicates, s2$Prob, r[s2$ID_unit, 
                             ]$Prob * s2$Prob)
@@ -250,12 +259,12 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
                   if (missing(method)) 
                     s1 = strata(result, stratanames = NULL, size = size[[j]], 
                       description = description)
-                  else if (method == "poisson" | method == "systematic") 
+                  else if (method[[j]] == "poisson" | method[[j]] == "systematic") 
                     s1 = strata(result, stratanames = NULL, size = size[[j]], 
-                      method = method, pik = pik[[j]], description = description)
+                      method = method[[j]], pik = pik[[j]], description = description)
                   else s1 = strata(result, stratanames = NULL, 
-                    size = size[[j]], method = method, description = description)
-                  if (method == "srswr") {
+                    size = size[[j]], method = method[[j]], description = description)
+                  if (method[[j]] == "srswr") {
                     s1 = cbind.data.frame(result[s1$ID_unit, 
                       ]$ID_unit, s1$Replicates, s1$Prob, result[s1$ID_unit, 
                       ]$Prob * s1$Prob)
@@ -284,4 +293,3 @@ mstage<-function (data, stage = c("stratified", "cluster", ""), varnames,
     names(res) = c(1:number)
     res
 }
-
